"use strict";
/**
 * @license
 * The MIT License (MIT)
 *
 * Copyright (c) 2022 Brandon Roberts
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateUrlsResolver = exports.StyleUrlsResolver = void 0;
exports.getStyleUrls = getStyleUrls;
exports.getTemplateUrls = getTemplateUrls;
const node_path_1 = require("node:path");
const ts_morph_1 = require("ts-morph");
const path_1 = require("path");
const utils_1 = require("./utils");
class StyleUrlsResolver {
    constructor() {
        // These resolvers may be called multiple times during the same
        // compilation for the same files. Caching is required because these
        // resolvers use synchronous system calls to the filesystem, which can
        // degrade performance when running compilations for multiple files.
        this.styleUrlsCache = new Map();
    }
    resolve(code, id) {
        // Given the code is the following:
        // @Component({
        //   styleUrls: [
        //     './app.component.scss'
        //   ]
        // })
        // The `matchedStyleUrls` would result in: `styleUrls: [\n    './app.component.scss'\n  ]`.
        const matchedStyleUrls = getStyleUrls(code)
            // for type narrowing
            .filter((v) => v !== undefined);
        const entry = this.styleUrlsCache.get(id);
        // We're using `matchedStyleUrls` as a key because the code may be changing continuously,
        // resulting in the resolver being called multiple times. While the code changes, the
        // `styleUrls` may remain constant, which means we should always return the previously
        // resolved style URLs.
        if (entry &&
            entry.matchedStyleUrls.join(',') === matchedStyleUrls.join(',')) {
            return entry.styleUrls;
        }
        const styleUrls = matchedStyleUrls.map((styleUrlPath) => {
            return `${styleUrlPath}|${(0, path_1.normalize)((0, node_path_1.resolve)((0, node_path_1.dirname)(id), styleUrlPath))}`;
        });
        this.styleUrlsCache.set(id, { styleUrls, matchedStyleUrls });
        return styleUrls;
    }
}
exports.StyleUrlsResolver = StyleUrlsResolver;
function getStyleUrls(code) {
    const project = new ts_morph_1.Project({ useInMemoryFileSystem: true });
    const sourceFile = project.createSourceFile('cmp.ts', code);
    const properties = sourceFile.getDescendantsOfKind(ts_morph_1.SyntaxKind.PropertyAssignment);
    const styleUrl = (0, utils_1.getTextByProperty)('styleUrl', properties);
    const styleUrls = (0, utils_1.getAllTextByProperty)('styleUrls', properties);
    return [...styleUrls, ...styleUrl];
}
function getTemplateUrls(code) {
    const project = new ts_morph_1.Project({ useInMemoryFileSystem: true });
    const sourceFile = project.createSourceFile('cmp.ts', code);
    const properties = sourceFile.getDescendantsOfKind(ts_morph_1.SyntaxKind.PropertyAssignment);
    return (0, utils_1.getTextByProperty)('templateUrl', properties);
}
class TemplateUrlsResolver {
    constructor() {
        this.templateUrlsCache = new Map();
    }
    resolve(code, id) {
        const entry = this.templateUrlsCache.get(id);
        if (entry?.code === code) {
            return entry.templateUrlPaths;
        }
        const templateUrlPaths = getTemplateUrls(code).map((url) => `${url}|${(0, path_1.normalize)((0, node_path_1.resolve)((0, node_path_1.dirname)(id), url).replace(/\\/g, '/'))}`);
        this.templateUrlsCache.set(id, { code, templateUrlPaths });
        return templateUrlPaths;
    }
}
exports.TemplateUrlsResolver = TemplateUrlsResolver;
